@@ -4,6 +4,7 @@ use crate::{EResult, Error};
 use elbus::rpc::{self, RpcClient, RpcHandlers};
 use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
+use std::ffi::CString;
 use std::fmt;
 #[cfg(feature = "extended-value")]
 use std::path::Path;
@@ -89,8 +90,6 @@ pub struct Initial {
     workers: u32,
     #[serde(default)]
     user: Option<String>,
-    #[serde(default)]
-    group: Option<String>,
 }
 
 impl Initial {
@@ -107,7 +106,6 @@ impl Initial {
         config: Option<&Value>,
         workers: u32,
         user: Option<&str>,
-        group: Option<&str>,
     ) -> Self {
         Self {
             config_version: SERVICE_CONFIG_VERSION,
@@ -122,7 +120,6 @@ impl Initial {
             config: config.map(Clone::clone),
             workers,
             user: user.map(ToOwned::to_owned),
-            group: group.map(ToOwned::to_owned),
         }
     }
     #[inline]
@@ -150,12 +147,13 @@ impl Initial {
         self.user.as_deref()
     }
     #[inline]
-    pub fn group(&self) -> Option<&str> {
-        self.group.as_deref()
-    }
-    #[inline]
-    pub fn data_path(&self) -> &str {
-        &self.data_path
+    pub fn data_path(&self) -> Option<&str> {
+        if let Some(ref user) = self.user {
+            if user == "nobody" {
+                return None;
+            }
+        }
+        Some(&self.data_path)
     }
     #[inline]
     pub fn timeout(&self) -> Duration {
@@ -297,29 +295,34 @@ impl Initial {
     }
     #[inline]
     pub fn drop_privileges(&self) -> EResult<()> {
-        if let Some(ref group) = self.group {
-            let g = get_system_group(group)?;
-            nix::unistd::setgroups(&[g.gid]).map_err(|e| {
-                Error::failed(format!(
-                    "Failed to switch the process groups to {}: {}",
-                    group, e
-                ))
-            })?;
-            nix::unistd::setgid(g.gid).map_err(|e| {
-                Error::failed(format!(
-                    "Failed to switch the process group to {}: {}",
-                    group, e
-                ))
-            })?;
-        }
         if let Some(ref user) = self.user {
             let u = get_system_user(user)?;
-            nix::unistd::setuid(u.uid).map_err(|e| {
-                Error::failed(format!(
-                    "Failed to switch the process user to {}: {}",
-                    user, e
-                ))
-            })?;
+            if nix::unistd::getuid() != u.uid {
+                let c_user = CString::new(user.as_str())
+                    .map_err(|e| Error::failed(format!("Failed to parse user {}: {}", user, e)))?;
+
+                let groups = nix::unistd::getgrouplist(&c_user, u.gid).map_err(|e| {
+                    Error::failed(format!("Failed to get groups for user {}: {}", user, e))
+                })?;
+                nix::unistd::setgroups(&groups).map_err(|e| {
+                    Error::failed(format!(
+                        "Failed to switch the process groups for user {}: {}",
+                        user, e
+                    ))
+                })?;
+                nix::unistd::setgid(u.gid).map_err(|e| {
+                    Error::failed(format!(
+                        "Failed to switch the process group for user {}: {}",
+                        user, e
+                    ))
+                })?;
+                nix::unistd::setuid(u.uid).map_err(|e| {
+                    Error::failed(format!(
+                        "Failed to switch the process user to {}: {}",
+                        user, e
+                    ))
+                })?;
+            }
         }
         Ok(())
     }
